package com.alastairbreeze.connectedworlds.Engine;

import static org.lwjgl.opengl.GL11.GL_LIGHTING;
import static org.lwjgl.opengl.GL11.glDisable;
import static org.lwjgl.opengl.GL11.glEnable;
import static org.lwjgl.opengl.GL11.glPopMatrix;
import static org.lwjgl.opengl.GL11.glPushMatrix;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Random;

import com.alastairbreeze.connectedworlds.Vec3;
import com.alastairbreeze.connectedworlds.Entities.Entity;
import com.alastairbreeze.connectedworlds.Entities.EntityAlive;
import com.alastairbreeze.connectedworlds.Entities.EntityBearFriendly;
import com.alastairbreeze.connectedworlds.Entities.EntityBearUnfriendly;
import com.alastairbreeze.connectedworlds.Entities.EntityHuman;
import com.alastairbreeze.connectedworlds.Entities.EntityHumanFriendly;
import com.alastairbreeze.connectedworlds.Entities.EntityHumanFriendlyChild;
import com.alastairbreeze.connectedworlds.Entities.EntityPandaFriendly;
import com.alastairbreeze.connectedworlds.Entities.EntityPandaUnfriendly;
import com.alastairbreeze.connectedworlds.Entities.EntityParticle;
import com.alastairbreeze.connectedworlds.Entities.EntityPlayer;
import com.alastairbreeze.connectedworlds.Entities.EntityRat;
import com.alastairbreeze.connectedworlds.Entities.EntityRatUnfriendly;
import com.alastairbreeze.connectedworlds.Entities.EntityTree;
import com.alastairbreeze.connectedworlds.Entities.EntityUnfriendly;
import com.alastairbreeze.connectedworlds.Entities.EntityUnfriendlyChild;
import com.alastairbreeze.connectedworlds.Entities.Items.EntityItem;
import com.alastairbreeze.connectedworlds.Entities.Items.EntityItemDagger;
import com.alastairbreeze.connectedworlds.Entities.Items.EntityItemHealth;
import com.alastairbreeze.connectedworlds.Entities.Items.EntityItemRegenerate;
import com.alastairbreeze.connectedworlds.Entities.Items.EntityItemSabre;
import com.alastairbreeze.connectedworlds.Entities.Items.EntityItemSword;
import com.alastairbreeze.connectedworlds.Meshes.Cube;
import com.alastairbreeze.connectedworlds.Meshes.Mesh;
import com.alastairbreeze.connectedworlds.Worlds.Island;
import com.alastairbreeze.connectedworlds.Worlds.World;

public class Game {

	private static Game gameInstance;
	public EntityPlayer player;
	public ArrayList<Entity> entities = new ArrayList<Entity>();
	public World currentWorld;
	public static Random rand = new Random();
	public ArrayList<EntityParticle> particles = new ArrayList<EntityParticle>();
	public boolean paused = false;
	public boolean home = false;
	public boolean rettomenu = false;
	public float cycle = 0f;
	public int runLeft = 60*2;
	public int level = 1;
	public boolean carryon = false;
	
	int guiCurrent = -1;
	int guiHome = -1;
	int guiGame = -1;
	int guiPause = -1;
	int guiFail = -1;
	int guiWin = -1;
	
	public static Game getInstance(){
		if(gameInstance==null){
			gameInstance = new Game();
			gameInstance.init();
		}
		return gameInstance;
	}
	
	public Game(){

	}
	
	public void init(){
		SoundManager.getInstance();
		currentWorld = new Island();
		currentWorld.init();
    	player = new EntityPlayer();
		player.init();
		double deg = rand.nextDouble()*2d*Math.PI;
		player.pos.x = 40f*(float)Math.sin(deg)+50f;
		player.pos.z = 40f*(float)Math.cos(deg)+50f;
//		player.pos.y = currentWorld.getHeight(player.pos.x, player.pos.z);
		player.pos.y = 10f;
		entities.clear();
		particles.clear();
		carryon = false;
		
//		EntityBear eb = new EntityBearFriendly();
//		eb.pos = player.pos.clone();
//		eb.init();
//		entities.add(eb);
	
		initGuis();
		home = true;
		paused = true;
		guiCurrent = guiHome;
		
		for(int i=0;i<50;i++){
			Entity e = new EntityTree();
			e.universe = -1;
			e.pos.x = (int)(float)(rand.nextFloat()*50f+25f);
			e.pos.z = (int)(float)(rand.nextFloat()*50f+25f);
			e.pos.y = currentWorld.getHeight(e.pos.x, e.pos.z);
			e.init();
			entities.add(e);
		}
		for(int i=0;i<10+2*level;i++){
			entitySpawner();
		}

		
//		int guiSuccess = -1;
//		for(int i=0;i<25;i++){
//			EntityItem ei = new EntityItemDagger();
//			ei.universe = rand.nextInt(2);
//			ei.pos.x = rand.nextFloat()*5f;
//			ei.pos.z = rand.nextFloat()*20f;
//			ei.init();
//			entities.add(ei);
//		}
	}
	
	public void initGuis(){
		guiHome = Core.startList();
		renderText(getHome());
		Core.endList();
		
		guiPause = Core.startList();
		renderText(getPause());
		Core.endList();
		
//		updateGameGui();
//		updateFailGui();
//		updateWinGui();
//		updateWinGui();
	}
	
	public void updateGameGui(){
		guiGame = Core.startList();
		renderText(getGame());
		Core.endList();
	}
	
	public void updateFailGui(){
		guiFail = Core.startList();
		renderText(getFail());
		Core.endList();
	}

	public void updateWinGui(){
		guiWin = Core.startList();
		renderText(getWin());
		Core.endList();
	}
	
	int neededLeft = -1;
	int curUni = -1;
	String itemName = "";
	public void update(){
		if(!paused){
			if(!player.alive){
				loose();
			}
			player.update();
			for(Entity e : entities){
				e.update();
			}
			for(EntityParticle p : particles){
				p.update();
			}
			int q = getNeededLeft();
			
			if(neededLeft!=q||curUni!=player.universe||!itemName.equals(player.currentItem.getName())){
				neededLeft = q;
				curUni = player.universe;
				itemName = player.currentItem.getName();
				Core.restartList(guiGame);
				renderText(getGame());
				Core.endList();
			}
			if(neededLeft == 0){
				win();
			}
//			entitySpawner();
			itemSpawner();
			currentWorld.update();
		}
	}
	
	public int countEntities(int uni){
		int i = 0;
		for(Entity e : entities){
			if (e.universe == uni && !(e instanceof EntityItem)  && !(e instanceof EntityTree)){
				i++;
			}
		}
		return i;
	}
	
	public void win(){
		updateWinGui();
		paused = true;
		rettomenu = false;
		home = false;
		carryon = true;
		guiCurrent = guiWin;
		player.pos.y = 10f;
		player.alive=false;
		level++;
	}
	
	public void loose(){
		updateFailGui();
		paused = true;
		rettomenu = false;
		guiCurrent = guiFail;
		player.pos.y = 10f;
		player.alive=false;
	}
	
	public Entity getRandEntityOfType(Class a){
		ArrayList<Entity> ls = new ArrayList<Entity>();
		for(Entity e: entities){
			if(e.getClass().equals(a)){
				ls.add(e);
			}
		}
		if(ls.size()==0){
			return null;
		}else{
			return ls.get(rand.nextInt(ls.size()));
		}
	}
	
	public void itemSpawner(){
		int i = rand.nextInt(2+10+4+1+1);
		int cou = 0;
		for(Entity e : entities){
			if(e instanceof EntityItem){
				cou++;
			}
		}
		if(cou < 20){
			EntityItem ent = null;
			if(i<2){
				ent = new EntityItemDagger();
			}else if(i<12){
				ent = new EntityItemHealth();
			}else if(i<16){
				ent = new EntityItemRegenerate();
			}else if(i<17){
				ent = new EntityItemSabre();
			}else if(i<18){
				ent = new EntityItemSword();
			}
			ent.pos.x = rand.nextFloat()*100f;
			ent.pos.z = rand.nextFloat()*100f;
			ent.pos.y = 0f;
			ent.universe = rand.nextInt(2);
			ent.init();
			entities.add(ent);
		}
	}
	
	public void entitySpawner(){
//		Uni1
//		if(countEntities(0) < 12+3*level){
		for(int un = 0;un<2;un++){
			int i = rand.nextInt(100+150+100+50+50+70+35+70+35+70+35+70+35);
			EntityAlive ent = null;
			if(i<100){
				ent = new EntityHumanFriendly();
			}else if(i<250){
				ent = new EntityRat();
			}else if(i<350){
				ent = new EntityUnfriendly();
			}else if(i<400){
				ent = new EntityHumanFriendlyChild((EntityHuman)getRandEntityOfType(EntityHumanFriendly.class));
			}else if(i<450){
				ent = new EntityUnfriendlyChild((EntityHuman)getRandEntityOfType(EntityUnfriendly.class));
			}else if(i<520){
				ent = new EntityBearFriendly();
			}else if(i<555){
				ent = new EntityBearFriendly();
				ent.scale = 0.5f;
			}else if(i<625){
				ent = new EntityBearUnfriendly();
			}else if(i<660){
				ent = new EntityBearUnfriendly();
				ent.scale = 0.5f;
			}else if(i<730){
				ent = new EntityPandaFriendly();
			}else if(i<800){
				ent = new EntityPandaUnfriendly();
			}else if(i<835){
				ent = new EntityPandaFriendly();
				ent.scale = 0.5f;
			}else if(i<870){
				ent = new EntityPandaUnfriendly();
				ent.scale = 0.5f;
			}else if(i<150){
				ent = new EntityRatUnfriendly();
			}
			ent.pos.x = rand.nextFloat()*100f;
			ent.pos.z = rand.nextFloat()*100f;
			ent.pos.y = 0f;
			ent.universe = un;
			ent.init();
			entities.add(ent);
		}
	}
	
	public int getNeededLeft(){
		int i=0;
		for(Entity e : entities){
			if(e.important){
				if(e.universe ==1 && e.good){
					i++;
				}
				if(e.universe ==0 && !e.good){
					i++;
				}
			}
		}
		return i;
	}

	
	public void render(){
		if(!paused){
			cycle = (System.currentTimeMillis()/10%360);
		}
		if(home||!player.alive){
			Core.c.camera.rot.y+=0.3f;
		}
		if(!home&&player.alive){
			glPushMatrix();
			{
	//			glLoadIdentity();
	//		Core.L("" + Core.c.camera.pos);
				Mesh.translate(player.pos, new Vec3(-Core.c.camera.rot.x*0f,-Core.c.camera.rot.y,-Core.c.camera.rot.z*0f));
	//			Mesh.translate(Vec3.add(new Vec3(Core.c.camera.pos.x,Core.c.camera.pos.y,Core.c.camera.pos.z),new Vec3(0f,-1.5f,0f)), new Vec3(-Core.c.camera.rot.x*0f,-Core.c.camera.rot.y,-Core.c.camera.rot.z*0f));
				Mesh.translate(new Vec3(3.5f*Core.c.camera.dist/7f,4.5f*Core.c.camera.dist/7f,0.01f), new Vec3(-Core.c.camera.rot.x-20f,0f-10f,Core.c.camera.rot.y));
				Cube.render(new Vec3(0f,0f,-0.150f), new Vec3(2f,2f,0.3f),0xffffff);
				glDisable(GL_LIGHTING);
				Core.renderPoint(new Vec3(player.pos.x/50f-1f,-player.pos.z/50f+1f,0.06f), 7f, 0x0000ff);
				for(Entity e:entities){
					if(e instanceof EntityAlive && !(e instanceof EntityItem) && (e.universe == player.universe || e.universe == -1)){
						int col = 0xff0000;
						EntityAlive ea = (EntityAlive)e;
						if(ea.good){
							col = 0x00ff00;
						}
						Core.renderPoint(new Vec3(e.pos.x/50f-1f,-e.pos.z/50f+1f,0.04f), 4f, col);
					}
				}
				glEnable(GL_LIGHTING);
			}
			glPopMatrix();
		}
		
		glPushMatrix();
		{
			Mesh.translate(player.pos, new Vec3(-Core.c.camera.rot.x*0f,-Core.c.camera.rot.y,-Core.c.camera.rot.z*0f));
			Mesh.translate(new Vec3(-3.5f*Core.c.camera.dist/7f,4.5f*Core.c.camera.dist/7f,0.01f), new Vec3(-Core.c.camera.rot.x,0f+0f,Core.c.camera.rot.y*0f));
			Core.renderList(guiCurrent);
		}
		glPopMatrix();
		if(player.alive&&!home){
			player.render();
		}
		ArrayList<Entity> deadEntities = new ArrayList<Entity>();
		for(Entity e : entities){
			if((e.universe == -1 || player.universe == e.universe)){
				if(e.alive){
					e.render();
				}else{
					deadEntities.add(e);
				}
			}
		}
		for(Entity p : deadEntities){
			if(p.important && p.good){
				loose();
			}
			p.die();
			entities.remove(p);
		}
		
		ArrayList<EntityParticle> deadParticles = new ArrayList<EntityParticle>();
		for(EntityParticle p : particles){
			if(p.alive){
				p.render();
			}else{
				deadParticles.add(p);
			}
		}
		for(EntityParticle p : deadParticles){
			particles.remove(p);
		}
		
	
		
		currentWorld.render();
	}
	
	public ArrayList<String> getHome(){
		ArrayList<String> ls = new ArrayList<String>();
		ls.add(Core.NAME);
		ls.add("");
		ls.addAll(getHelp());
		ls.add("");
		ls.addAll(getControls());
		ls.add("");
		ls.add("Press Space To Start");
		ls.add("");
		ls.addAll(getCopyright());
		return ls;
	}
	
	public ArrayList<String> getGame(){
		ArrayList<String> ls = new ArrayList<String>();
		String un = "Good";
		if(curUni==1){
			un = "Evil";
		}
		ls.add("Level:"+level);
		ls.add("Universe:"+un);
		ls.add("Holding:"+itemName);
		ls.add("Remaining:"+neededLeft);
		return ls;
	}
	
	public ArrayList<String> getPause(){
		ArrayList<String> ls = new ArrayList<String>();
		ls.add("PAUSED");
		ls.add("Press P to un-pause");
		ls.add("Press R to exit to menu");
		ls.add("");
		ls.addAll(getControls());
		return ls;
	}
	
	public ArrayList<String> getFail(){
		ArrayList<String> ls = new ArrayList<String>();
		ls.add("FAILED");
		ls.add("You failed too keep");
		ls.add("the good alive!");
		ls.add("");
		ls.add("You had "+neededLeft + " left");
		ls.add("to swap.");
		ls.add("");
		ls.add("You reached level "+level+".");
		ls.add("");
		ls.add("Press R to go");
		ls.add("to the main menu.");
		return ls;
	}
	
	public ArrayList<String> getWin(){
		ArrayList<String> ls = new ArrayList<String>();
		ls.add("SUCCESS");
		ls.add("Congrats, you completed");
		ls.add("level "+level+".");
		ls.add("");
		ls.add("Press space to go");
		ls.add("to carry on.");
		return ls;
	}
	
	public ArrayList<String> getHelp(){
		ArrayList<String> ls = new ArrayList<String>();
		ls.add("2 Parralel Universes");
		ls.add("have collided.");
		ls.add("The evil universe is now");
		ls.add("mixed with the good.");
		ls.add("Use the switcher item");
		ls.add("to switch everyone back.");
		ls.add("Dont let the good die!");
		return ls;
	}
	
	public ArrayList<String> getControls(){
		ArrayList<String> ls = new ArrayList<String>();
		ls.add("Controls");
		ls.add("WASD      :Move");
		ls.add("E/L-Click :Use Item");
		ls.add("Q/R-Click :Change Item");
		ls.add("Shift     :Sprint");
		ls.add("Arrows    :Camera");
		ls.add("P         :Pause");
		ls.add("M         :Toggle Mute");
		return ls;
	}
	
	public ArrayList<String> getCopyright(){
		ArrayList<String> ls = new ArrayList<String>();
		ls.add("Copyright (c)");
		ls.add("Alastair Breeze 2014");
		ls.add("www.alastairbreeze.com");
		return ls;
	}
	
	public void renderText(ArrayList<String> lines){
		glPushMatrix();
		{
			glDisable(GL_LIGHTING);
			
//			Mesh.translate(player.pos, new Vec3(-Core.c.camera.rot.x*0f,-Core.c.camera.rot.y,-Core.c.camera.rot.z*0f));
//			Mesh.translate(new Vec3(-3.5f*Core.c.camera.dist/7f,4.5f*Core.c.camera.dist/7f,0.01f), new Vec3(-Core.c.camera.rot.x,0f+0f,Core.c.camera.rot.y*0f));

			BufferedImage img = new BufferedImage(400,20*(lines.size()+1),BufferedImage.TYPE_INT_ARGB);
			Graphics2D g = img.createGraphics();
			g.setFont(new Font("Lucida Console",Font.BOLD, 20));
			
			int i=1;
			for(String s:lines){
				g.drawString(s,0,i*20);
				i++;
			}
			
			for(int x=0;x<img.getWidth();x+=1){
				for(int y=0;y<img.getHeight();y+=1){
					if(img.getRGB(x, y)!=0){
						Core.renderPoint(new Vec3((float)x/70f-1f,-(float)y/70f+1f,0.06f), 1.3f, img.getRGB(x, y));
					}
				}
			}
			
			glEnable(GL_LIGHTING);
		}
		glPopMatrix();
	}
	
	public void particleExplosion(Vec3 p, int col, float v){
		int z = (int)(v*10000f);
		for(int i=0;i<300;i++){
			Vec3 v1 = new Vec3(rand.nextFloat()-0.5f,rand.nextFloat()-0.5f,rand.nextFloat()-0.5f);
			v1.normalize();
			v1.scale(v/50f);
			EntityParticle part = new EntityParticle(p,v1,1000l, col);
			part.init();
			particles.add(part);
		}
	}
}
