package com.alastairbreeze.connectedworlds.Entities;

import static org.lwjgl.opengl.GL11.GL_LIGHTING;
import static org.lwjgl.opengl.GL11.glDisable;
import static org.lwjgl.opengl.GL11.glEnable;
import static org.lwjgl.opengl.GL11.glPopMatrix;
import static org.lwjgl.opengl.GL11.glPushMatrix;

import java.util.ArrayList;

import com.alastairbreeze.connectedworlds.Vec3;
import com.alastairbreeze.connectedworlds.Engine.Core;
import com.alastairbreeze.connectedworlds.Engine.Game;
import com.alastairbreeze.connectedworlds.Engine.SoundManager;
import com.alastairbreeze.connectedworlds.Entities.Items.EntityItem;
import com.alastairbreeze.connectedworlds.Entities.Items.EntityItemHand;
import com.alastairbreeze.connectedworlds.Meshes.Cube;

public class EntityAlive extends Entity{
	public Vec3 vel = new Vec3();
	public Vec3 acc = new Vec3();
	
	public float attackAni = 0f;

	public float speed = 0.0025f;
	public float moveSpeed = 0.008f;
	
	public int health = 1;
	public int maxHealth = 1;
	public long lastHit = 0l;
	
	public int points = 0;
	
	boolean grounded = true;
	
	boolean collidable = true;
	
	public boolean run = false;
	public int maxRunLeft = 60*2;
	public int runLeft = maxRunLeft;
	public float runScale = 1.85f;
	
	public Vec3 target = null;
	public long pauseTill = 0l;
	
	float sight = 4f;
	
	ArrayList<EntityItem> items = new ArrayList<EntityItem>();
	public EntityItem currentItem = null;
	
	public EntityAlive(){
		super();
	}
	
	public void init(){
		super.init();
		hold(new EntityItemHand());
	}
	
	public void update(){
		updateHealth();
		if(!alive)return;
		super.update();
		
		acc.scale(0.85f);
		vel.scale(0.85f);
		
		if(run){
			runLeft--;
			if(runLeft < 0){
				runLeft = 0;
				run=false;
			}
		}else{
			runLeft++;
			if(runLeft > maxRunLeft){
				runLeft = maxRunLeft;
			}
		}
		
		if(rot!=null){
			float tarRY = rot.y;
			if(vel.z > 0.001f){
				tarRY = (float)Math.toDegrees(Math.atan(vel.x/vel.z));
			}else if(vel.z<=-0.001f){
				tarRY = -180f+(float)Math.toDegrees(Math.atan(vel.x/vel.z));
			}
			rot.y = tarRY;
		}
		
		float tarY = Game.getInstance().currentWorld.getHeight(pos.x, pos.z);
		if(tarY < pos.y){
			acc.y = -9.8f/350f;
		}
	
		Game.getInstance().currentWorld.inBounds(this);
		
		vel.add(acc);
		pos.add(vel);
		if(pos.y < tarY){
//			if(tarY - pos.y < 4f){
				pos.y = tarY;
				grounded =true;
//			}else{
//				Vec3 v = new Vec3(new Vec3(Math.round(pos.x),Math.round(pos.y),Math.round(pos.z)),pos);
//				v.normalize();
//				v.scale(0.1f);
//				pos.add(v);
//				collide();
//				grounded = false;
//				if(this == Game.getInstance().player){
////					jump();
//				}
//			}
		}
		
		attackAni*=0.9f;
		
		if(collidable){
			checkCollisions();
		}
		ArrayList<EntityItem> usedItems = new ArrayList<EntityItem>();
		for(EntityItem ei : items){
			if(ei!=null){
				if(ei.used){
					usedItems.add(ei);
				}
			}
		}
		for(EntityItem ei : usedItems){
			if(currentItem == ei){
				nextItem();
			}
			items.remove(ei);
		}
	}
	
	public void updateHealth(){
		if(health < 0){
			health = 0;
			alive = false;
		}else if(health > maxHealth){
			health = maxHealth;
		}
	}
	
	public void checkCollisions(){
		for(Entity e : Game.getInstance().entities){
			if(e!=this && (e.universe == universe || e.universe == -1)){
				if(checkCollision(e.pos,e.rad)){
					if(e instanceof EntityItem){
						((EntityItem)e).pickup(this);
					}else{
						Vec3 v = new Vec3(e.pos,pos);
						v.scale(0.05f);
						vel.add(v);
						collide();
					}
				}
			}
		}
	}
	
	public boolean canSee(Entity e, float dist){
		if(e!=this){
			if(e.universe == universe || e.universe == -1 || universe == -1){
				if(new Vec3(e.pos,this.pos).getMag() < dist){
					return true;
				}
			}
		}
		return false;
	}
	
	public boolean checkCollision(Vec3 p, float r){
		if(Vec3.getDist(pos,p) < rad + r){
			return true;
		}
		return false;
	}
	
	public void stop(){
		vel = new Vec3();
		acc = new Vec3();
	}
	
	public void collide(){
		
	}
	
	public void jump(){
		if(grounded){
			acc.y += 0.6f;
			grounded = false;
		}
		
	}
	
	public void hold(EntityItem e){
		if(e.repeatable){
			items.add(e);
			currentItem = e;
		}else{
			for(EntityItem ei : items){
				if(e!=null && ei!=null){
					if(ei.getClass().equals(e.getClass())){
						return;
					}
				}else if(e==null && ei==null){
					return;
				}
			}
			items.add(e);
			currentItem = e;
		}
		if(this!=Game.getInstance().player){
			useItem();
		}
	}
	
	long lastUse = 0l;
	public void useItem(){
		long reTi = 250l;
		if(currentItem!=null){
			reTi = currentItem.reuseTime;
		}
		if(System.currentTimeMillis() - lastUse > reTi){
			if(currentItem!=null){
				currentItem.use(this);
			}else{
				attack(1f);
			}
			lastUse = System.currentTimeMillis();
		}
	}
	
	public EntityAlive getEntityInfront(float sig){
		if(this!=Game.getInstance().player && canSee(Game.getInstance().player,sig)){
			return Game.getInstance().player;
		}
		ArrayList<EntityAlive> ls = new ArrayList<EntityAlive>(); 
		for(Entity e : Game.getInstance().entities){
			if(e instanceof EntityAlive){
				EntityAlive ea = (EntityAlive)e;
				if(canSee(ea,sight)){
					ls.add(ea);
				}
			}
		}
		EntityAlive ret = null;
		float dist = 100f;
		for(EntityAlive e : ls){
			float x = new Vec3(e.pos,this.pos).getMag();
			if(x < dist){
				dist = x;
				ret = e;
			}
		}
		return ret;
	}
	
	public void attack(float f){
		float mag = 2f;
		if(currentItem!=null){
			mag = currentItem.range;
		}
		EntityAlive ae = getEntityInfront(mag);
		attackAni = 1f;
		if(ae!=null){
			int i = (int)(f/2f);
			if(i<1)i=1;
			ae.hurt((int)f + Game.getInstance().rand.nextInt(i),this);
			if(rot!=null){
				Vec3 v = new Vec3((float)Math.sin(Math.toRadians((double)rot.y)),0f,(float)Math.cos(Math.toRadians((double)rot.y)));
				v.scale(f/25f);
				ae.vel.add(v);
			}
		}
	}
	
	public void hurt(int a,EntityAlive ea){
		if(ea!=null&&good){
			Vec3 v = new Vec3(ea.pos,pos);
			v.normalize();
			v.scale(20f);
			target = Vec3.add(pos,v);
			resetPauseTill();
			run = true;
		}
		if(new Vec3(ea.pos,Game.getInstance().player.pos).getMag()<5f){
			SoundManager.getInstance().playHurtSound();
		}
		health -= a;
	}
	
	public void heal(int a){
		health += a;
	}
	
	public void nextItem(){
//		if(currentItem != null){
			int index = items.indexOf(currentItem);
			index--;
			if(index >= 0){
				currentItem = items.get(index);
			}else{
				currentItem = items.get(items.size()-1);
			}
//		}else{
//			if(items.size()>0){
//				currentItem = items.get(0);
//			}
//		}
	}
	
	public void findEnemy(){
		EntityAlive e = getEntityInfront(sight);
		if(e!=null){
			if(e.good){
				target = e.pos.clone();
				this.useItem();
			}
		}
	}
	
	public void randTarget(){
		if(target == null || System.currentTimeMillis() > pauseTill){
			target = new Vec3(Game.getInstance().rand.nextFloat()*100f,0,Game.getInstance().rand.nextFloat()*100f);
			resetPauseTill();
		}
	}
	
	public void followTarget(){
		if(target!=null){
			Vec3 v = new Vec3(pos,target);
			if(v.getMagXZ() > 1.5f){
				v.normalize();
				if(run && runLeft > 0){
					v.scale(moveSpeed*runScale);
				}else{
					v.scale(moveSpeed);
				}
				acc = v;
			}else{
				resetPauseTill();
				target = null;
			}
		}
	}
	
	public boolean canRun(){
		return (run && runLeft > 0);
	}
	
	public void resetPauseTill(){
		pauseTill = System.currentTimeMillis() + (long)Game.getInstance().rand.nextInt(10000);
	}
	
	public void die(){
		super.die();
		for(EntityItem item : items){
			item.alive = true;
			item.pos = pos.clone();
			item.universe = universe;
//			item.pos.add(new Vec3(Game.getInstance().rand.nextFloat()*2f - 1f,0f,Game.getInstance().rand.nextFloat()*2f - 1f));
			if(!(item instanceof EntityItemHand) && item != null){
				Game.getInstance().entities.add(item);
			}
		}
	}
	
	public void render(){
		super.render();
		if(health!=maxHealth){
			glPushMatrix();
			{
				glDisable(GL_LIGHTING);
				int col = 0;
				int g = (int)(255f*(float)health/(float)maxHealth);
				int r = 255 - g;
				col = (0xff<<24) | ((0xff&r)<<16) | ((0xff&g)<<8) | 0x00;
				
				Cube.render(Vec3.add(pos, new Vec3(0f,height + 0.6f,0f)), new Vec3((float)health/(float)maxHealth,0.1f,0.1f),new Vec3(rot.x,-Core.c.camera.rot.y,rot.z), col);
				glEnable(GL_LIGHTING);
			}
			glPopMatrix();
		}
	}
}
